#!/bin/sh

prefix=/opt/drweb.com
libdir=/opt/drweb.com/lib64/samba
scriptdir=$prefix/share/drweb-smbspider-modules
me=$0

# Checks if string is in the list.
#
# $1 - list
# $2 - string
StringInList() {
    if  (echo $1 | grep "^$2[[:space:]]" >/dev/null) ||
        (echo $1 | grep "[[:space:]]$2\$" >/dev/null) ||
        (echo $1 | grep "^$2\$" >/dev/null) ||
        (echo $1 | grep "[[:space:]]$2[[:space:]]" >/dev/null); then
        return 0
    else
        return 1
    fi
}

GetArch() {
    out="`isalist 2>/dev/null``uname -m 2>/dev/null`"
    case $out in
	*amd64*|*x86_64*)
            echo "amd64" ;;
	*i*86*)
            echo "i386" ;;
        *)
            echo "$out" ;;
    esac
}

find_smbd()
{
    bins=""
    for p in /opt/csw /usr /usr/local /usr/local/samba /opt/samba ; do
        bins="$bins $p/*bin/smbd"
    done

    echo "$bins"
}

get_smbd_version()
{
    $1 -V | sed 's/Version //'
}

get_smbd_libdir()
{
    # at first use MODULESDIR (for samba >= 3.3) else (if not found) use LIBDIR
    ($1 -b | grep MODULESDIR || $1 -b | grep LIBDIR) | sed 's/.*DIR: //'
}

get_smbd_config()
{
    $1 -b | grep CONFIGFILE | sed 's/.*CONFIGFILE: //'
}

# Check if smbd and vfs module are compatible.
module_compatible()
{
    if  echo $smbd_version | grep '^3\.2' >/dev/null 2>&1 || \
        echo $smbd_version | grep '^3\.3' >/dev/null 2>&1 || \
        echo $smbd_version | grep '^3\.4' >/dev/null 2>&1 || \
        echo $smbd_version | grep '^3\.5' >/dev/null 2>&1; then

        if [ "`GetArch`" = "i386" ]; then
            if [ "$smbd_version" = "3.5.6" ]; then
                module_clust=1
            else
                module_clust=
            fi
            if $1 -b | grep CLUSTER_SUPPORT >/dev/null 2>&1; then
                smbd_clust=1
            else
                smbd_clust=
            fi

            if [ "$module_clust" != "$smbd_clust" ]; then
                if [ -z "$smbd_clust" ]; then
                    reason=without
                else
                    reason=with
                fi
                cat <<EOF

Warning: The precompiled VFS module can't be used with $1
         because the latter has been built $reason the cluster support.
         You can build the VFS module from the source though.
EOF
                return 1
            fi
        fi
    fi

    return 0
}

# update links for one Samba
update_for()
{
    # echo
    # echo "--------------------------------------------------------"
    # echo "Update links for" $1;
    smbd_version=`get_smbd_version $1`;
    smbd_libdir=`get_smbd_libdir $1`/vfs;
    smbd_config=`get_smbd_config $1`;

    if [ -d "$smbd_libdir" -o -h "$smbd_libdir" ] ; then
	true
    else
	echo "Directory for VFS module '$smbd_libdir' doesn't exist"
	return 1
    fi

    module_compatible $1 || return

    num=`sh $scriptdir/vfs-versions.sh $smbd_version`;
    if [ -n "$num" ] ; then
	if [ -h $smbd_libdir/smb_spider.so ] ; then
	    echo "Found old symlink in $smbd_libdir/smb_spider.so. Removing..."
	    rm -f $smbd_libdir/smb_spider.so
	fi
	if [ -f $smbd_libdir/smb_spider.so ] ; then
	    echo "$smbd_libdir/smb_spider.so is a regular file. Cannot create symlink."
	    return 1
	fi
	echo "create symlink" $libdir/libsmb_spider.so.$num "-->" $smbd_libdir/smb_spider.so;
	if (ln -s $libdir/libsmb_spider.so.$num $smbd_libdir/smb_spider.so);then
            if [ -z "$DR_INTERACTIVE" ]; then
	        echo "Please, update your config" $smbd_config;
	        echo "--------------------------------------------------------"
            fi
	else
	    echo $me: "Failed to create link!" 1>&2;
	    return 1
	fi
    else
	echo $me: "Could not determinate SAMBA version" 1>&2
	exit 1;
    fi
}

remove_for()
{
    so_link=`get_smbd_libdir $1`/vfs/smb_spider.so

    if ([ -f $so_link ] || [ -h $so_link ]);then
	echo "Remove link" $so_link;
	rm $so_link && [ -z "$DR_INTERACTIVE" ] && \
            echo "Please, update your config" `get_smbd_config $1`;
    fi
}

print_help()
{
    echo "Create links for smb_spider.so"
    echo "Usage:" $0 "[options]" "[smbd pathes]";
    printf "%-20s%s\n" "-h, --help" "show help";
    printf "%-20s%s\n" "-r, --remove" "remove links";
    printf "%-20s%s\n" "-s, --smbd-path" "for the chosen smbd only";
}

smbd_list="";
action="";
installed_any="";
found_any="";

# get options
while [ x"$1" != x"" ] 
do 
    case $1 in
	--help | -h) print_help; exit 1;;
	--remove | -r) action="remove";;
	--smbd-path | -s) shift; smbd_list=`echo $smbd_list $1`;;
	-*) echo "unknown option:" $1 1>&2; exit 1;;
	*) smbd_list=`echo $smbd_list $1`;;
    esac
    shift;
done

# get smbd list
if [ x"$smbd_list" = x"" ];then
    smbd_list=`find_smbd`
fi

# choose and do action
if [ x"$action" = x"remove" ];then
    for i in $smbd_list;do
	[ -x $i ] && remove_for $i;
    done
    exit 0;
else
    for i in $smbd_list;do
	[ -x "$i" ] && { found_any="yes"; } && update_for $i && { installed_any="yes"; };
    done

    if [ x"$found_any" != x"yes" ];then
	echo $me: "cannot find any smbd. Please run $me using '--smbd-path' parameter" 1>&2
    fi

    if [ x"$installed_any" = x"yes" ] && [ -z "$DR_INTERACTIVE" ]; then
	echo
	echo "Example:"
	echo "[drweb_audit]"
	echo "   comment = Directory protected by SpIDer Guard for SMB"
	echo "   path = /mnt/shares/protected_stuff"
	echo "   vfs objects = smb_spider"
    fi
fi
